# Literate Metadata 

## About the Literate file formats
The literate metada format is a [CommonMark](http://spec.commonmark.org/) document that has embedded code 
blocks for machine readable sections. This encourages easy-to-author documentation while permitting the specificity 
desired when authoring and processing instructions.

## Purpose
A markdown document that captures the current state of the SDK and presents it in an easy-to-consume format.

As much information in the Literate Metadata document should be autogenerated as possible from the CI system 

## Contributions
Mark provided the basis for content in the Metadata format, See the [original spec](https://microsoft.sharepoint.com/teams/CPT/AzureDevExXplat/_layouts/15/WopiFrame.aspx?sourcedoc=%7BEF3980B2-390A-41C8-8073-12E55B1468A1%7D&file=Rest%20Specs%20Proposal.docx&action=default )

### changes from the original spec.
In the original spec, the intention was to store metadata down the tree along side the swagger specs themselves:

``` yaml
# Repo structure (single spec) 
<RP-Name>/<api-version>/swagger.json        # swagger
<RP-Name>/<api-version>/swagger.md          # metadata

# Repo structure (composite spec) 
<RP-Name>/composite.<api-version>.json      # Composite swagger
<RP-Name>/<api-version>/<ResourceType>.json # swagger
<RP-Name>/<api-version>/<ResourceType>.md   # metadata
```

I suggest collapsing all the data and bubbling it up to the top level and placing at the same point as the Literate Configuration file.


``` yaml
# Repo structure (single spec) 
<RP-Name>/<api-version>/readme.md             # Literate Configuration file
<RP-Name>/<api-version>/swagger.json          # swagger file
<RP-Name>/<api-version>/meta-data.md          # metadata document 

# Repo structure (composite spec) 
<RP-Name>/readme.md                           # Literate configuration
<RP-Name>/metadata.md                         # Metadata document (for all)
<RP-Name>/<api-version>/<ResourceType>.json   # one swagger
<RP-Name>/<api-version>/<ResourceType2>.json  # another swagger
```

### Notable Features

- Blocks to insert CI/CD-generated metadata for SDK and CLI dashboards
- Include changelogs and approvers 

## The File Format


> Example: a markdown metadata file

``` markdown 
# Some ARM Resource
Some documentation text about an ARM resource

# State of the specifications stored here.

|Spec   |Version    |Complete|Correct|Compliant|API GA|Composition|Tag|
|-------|-----------|--------|-------|---------|------|-----------|---|
|Compute| 2015-05-01|Yes     |Yes    |No       |Yes   |Composite  |http://foo/bar/bin/baz|
|Compute| 2015-06-01|Yes     |Yes    |No       |Yes   |Composite  |http://foo/bar/bin/bar|
|Storage| 2015-09-01|Yes     |Yes    |Yes      |Yes   |Single     |http://foo/bar/bin/baz|

# Shipping State 

## SDK Dashboard 

|Spec   |API Version|.Net    |Java    | Python | Node   | Ruby   | Go     |
|-------|-----------|--------|--------|--------|--------|--------|--------|
|Compute| 2015-05-01|Stable  |None    |Preview |Preview |Preview |None    |
|Compute| 2015-06-01|Preview |None    |None    |Preview |None    |None    |

## CLI Dashboard
|Spec   |API Version|PowerShell |Python CLI|Node CLI |
|-------|-----------|-----------|----------|---------|
|Compute| 2015-05-01|Stable     |None      |None     |
|Compute| 2015-06-01|Preview    |None      |Preview  |

## Links

### .NET 
Published NuGet Packages : 
  - [2015-05-01 / Version 2.9.6](http://nuget.org/foo....)
  - [2015-05-01 / Version 2.9.5](http://nuget.org/foo....)
  - [2015-05-01 / Version 2.9.4](http://nuget.org/foo....)

### Python:
Published Python Packages 
  - [2016/05/01/ Version 1.0.7](http://python/pypy/foo....)

## Version History
(some versioning/historical data?)

## ChangeLog
- [2016/01/17] - jobader - initial spec
- [2016/01/18] - jobader - renamed elements for better readability.
- [2016/01/19] - jobader - added XML support.
```